import 'dart:convert';
import 'dart:io';

import 'package:file_picker/file_picker.dart';
import 'package:flutter/cupertino.dart';
import 'package:image_picker/image_picker.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_core/ui/component/nui_toast.dart';
import 'package:nui_core/ui/nui_navigator.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_enum.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_message.dart';
import 'package:path/path.dart';

class NUIMiniProgramActionUtil{

  Future<NUIMiniProgramMessage> processAction(BuildContext context, String message, NUIMiniProgramMessage msg, {Function initializeListener}) async{
    try {
      final internalMessage = NUIMiniProgramInternalMessage().fromJson(jsonDecode(message));
      switch(internalMessage.type){
        case NUIMiniProgramEventType.POP: {
          pop(context);
          return null;
        }
        case NUIMiniProgramEventType.TOAST: {
          toast(message, context: context);
          return null;
        }
        case NUIMiniProgramEventType.FILE_SELECT: {
          final selectedFile = await _selectFile();
          return msg.response(data: selectedFile);
        }
        case NUIMiniProgramEventType.HIDE_KEYBOARD: {
          FocusScope.of(context).unfocus();
          return null;
        }
        case NUIMiniProgramEventType.STATUS_BAR_LIGHT: {
          NUIDisplayUtil.setStatusBarStyleAttr(NUIOverlayStyle(darkStatusBarIcon: false), internal: true);
          return null;
        }
        case NUIMiniProgramEventType.STATUS_BAR_DARK: {
          NUIDisplayUtil.setStatusBarStyleAttr(NUIOverlayStyle(darkStatusBarIcon: true), internal: true);
          return null;
        }
        case NUIMiniProgramEventType.INITIALIZED: {
          if(initializeListener != null) initializeListener();
          return null;
        }
        case NUIMiniProgramEventType.CAMERA: {
          final selectedFile = await _takePhoto(rear: match(internalMessage.data, "true"));
          return msg.response(data: selectedFile);
        }
        case NUIMiniProgramEventType.VIDEO: {
          final selectedFile = await _captureVideo(rear: match(internalMessage.data, "true"));
          return msg.response(data: selectedFile);
        }
      }
    }catch(e, s){
      logNUI("NUIMiniProgramActionUtil", "Failed to parse NUIMiniProgramInternalMessage with error : $e, stacktrace: $s");
    }
    return null;
  }


  Future<NUIMiniProgramFile> _selectFile() async{
    FilePickerResult result = await FilePicker.platform.pickFiles(
      // allowedExtensions: ['jpg', 'pdf', 'doc', 'png'],
    );

    if(result != null) {
      final selected = result.files[0];
      print("NUIMiniProgramActionUtil: Selected file : $selected, filename: ${selected?.name}, path : ${selected?.path}, bytes: ${selected?.bytes?.length}");
      if(selected?.bytes != null) {
        return NUIMiniProgramFile(filename: selected.name, bytes: selected.bytes);
      }
      else{
        try{
          final file = File(selected.path);
          final bytes = await file.readAsBytes();
          return NUIMiniProgramFile(filename: selected.name, bytes: bytes);
        }catch(e, s){
          print("NUIMiniProgramActionUtil: Processed bytes failed with error: $e, stacktrace: $s");
        }
        return null;
      }
    } else {
      return null;
    }
  }

  Future<NUIMiniProgramFile> _takePhoto({bool rear: true}) async{
    final pickedFile = await ImagePicker().getImage(source: ImageSource.camera, preferredCameraDevice: rear ? CameraDevice.rear : CameraDevice.front);

    if (pickedFile != null) {
      try {
        final file = File(pickedFile.path);
        final bytes = await file.readAsBytes();
        return NUIMiniProgramFile(filename: basename(file.path), bytes: bytes);
      }catch(e, s){
        print("NUIMiniProgramActionUtil: Processed take photo bytes failed with error: $e, stacktrace: $s");
      }
    }
    return null;
  }

  Future<NUIMiniProgramFile> _captureVideo({bool rear: true}) async{
    final pickedFile = await ImagePicker().getVideo(source: ImageSource.camera, preferredCameraDevice: rear ? CameraDevice.rear : CameraDevice.front);

    if (pickedFile != null) {
      try {
        final file = File(pickedFile.path);
        final bytes = await file.readAsBytes();
        return NUIMiniProgramFile(filename: basename(file.path), bytes: bytes);
      }catch(e, s){
        print("NUIMiniProgramActionUtil: Processed capture video bytes failed with error: $e, stacktrace: $s");
      }
    }
    return null;
  }
}